Xts128

Struct Xts128 

pub struct Xts128<C> { /* private fields */ }
Xts128 block cipher. Does not implement implement BlockMode due to XTS differences detailed here.

Implementations

Source

impl<C> Xts128<C>
where C: BlockSizeUser<BlockSize = U16>,

pub fn new(cipher_1: C, cipher_2: C) -> Xts128<C>

Creates a new Xts128 using two cipher instances: the first one’s used to encrypt the blocks and the second one to compute the tweak at the start of each sector.

Usually both ciphers are the same algorithm, the key is stored as double sized (256 bits for Aes128). When using, the key is split in half, the first half used for cipher_1 and the other for cipher_2.

If you require support for different cipher types, or block sizes different than 16 bytes, open an issue.

Source

impl<C> Xts128<C>
where C: BlockSizeUser<BlockSize = U16> + BlockCipherEncrypt,

pub fn encrypt_sector(&self, sector: &mut [u8], tweak: Array<u8, U16>)

Encrypts a single sector in place using the given tweak.

Panics
  • If there’s less than a single block in the sector.
pub fn encrypt_area( &self, area: &mut [u8], sector_size: usize, first_sector_index: u128, get_tweak_fn: impl Fn(u128) -> Array<u8, U16>, )

Encrypts a whole area in place, usually consisting of multiple sectors.

The tweak is computed at the start of every sector using get_tweak_fn(sector_index). get_tweak_fn is usually get_tweak_default.

Panics
  • If there’s less than a single block in the last sector.
Source

impl<C> Xts128<C>

pub fn decrypt_sector(&self, sector: &mut [u8], tweak: Array<u8, U16>)

Decrypts a single sector in place using the given tweak.

Panics
  • If there’s less than a single block in the sector.
pub fn decrypt_area( &self, area: &mut [u8], sector_size: usize, first_sector_index: u128, get_tweak_fn: impl Fn(u128) -> Array<u8, U16>, )

Decrypts a whole area in place, usually consisting of multiple sectors.

The tweak is computed at the start of every sector using get_tweak_fn(sector_index). get_tweak_fn is usually get_tweak_default.

Panics
  • If there’s less than a single block in the last sector.